//
//  PrinterManager.m
//  PrinterExample
//
//  Created by King on 12/12/2017.
//  Copyright © 2017 Printer. All rights reserved.
//

#import "PrinterManager.h"

#import "WIFIFactory.h"
#import "PinPrinterFactory.h"
#import "ESCFactory.h"
#import "ThermalPrinter.h"
#import "ThermalPrinterFactory.h"
#import "LabelPrinterFactory.h"
#import "PinCmdFactory.h"
#import "BlueToothPI.h"
#import "BlueToothFactory.h"
#import "BlueToothPI.h"
#import "TSCFactory.h"
#import "CPCLFactory.h"
#import "LabelCommonSetting.h"



@implementation PrinterManager

+ (PrinterManager *)sharedInstance{
    static PrinterManager *_sharedInstance = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        _sharedInstance = [[PrinterManager alloc] init];
        _sharedInstance.CurrentPrinterPortType = PrinterPortWifi;
        _sharedInstance.CurrentPrinterCmdType = PrinterCmdESC;
        _sharedInstance.CurrentPrinter = [_sharedInstance CreatePrinterClass:PrinterCmdESC];
        _sharedInstance.PrinterList = [SafeMutableArray arrayWithCapacity:4];
        
    });
    
    return _sharedInstance;
}

-(Cmd *)CreateCmdClass:(PrinterCmdType)PrinterCmdType{
    Cmd *cmd;
    switch (PrinterCmdType) {
        case PrinterCmdESC:
            cmd =  [ESCFactory Create];
            break;
        case PrinterCmdTSC:
            cmd=  [TSCFactory Create];
            break;
        case PrinterCmdCPCL:
            cmd = [CPCLFactory Create];
            break;
        case PrinterCmdPIN:
            cmd = [PinCmdFactory Create];
            break;
            
        default:
            cmd=nil;
    }
    return cmd;
}


-(Printer *)CreatePrinterClass:(PrinterCmdType)PrinterCmdType{
    Printer * _printer;
    switch (PrinterCmdType) {
        case PrinterCmdESC:
            _printer =  [ThermalPrinterFactory Create];
            break;
        case PrinterCmdTSC:
        case PrinterCmdCPCL:
            _printer =  [LabelPrinterFactory Create];
            break;
        case PrinterCmdPIN:
            _printer = [PinPrinterFactory Create];
            break;
        default:
            _printer=nil;
    }
    return _printer;
}


-(void)DoConnectwifi:(NSString *)Address Port:(NSInteger)Port {
    PrinterInterface *printerInter = [WIFIFactory Create];
    printerInter.Address = Address;
    printerInter.Port = Port;
    printerInter.printerCmdtype = self.CurrentPrinterCmdType;
    [_CurrentPrinter setPrinterPi:printerInter];
    [_CurrentPrinter Open];
}




-(void)DoConnectBle:(NSString *)Address {
    BlueToothPI *blueToothPI =  [BlueToothFactory Create:BlueToothKind_Ble];
    blueToothPI.MTULength= 20;
    blueToothPI.SendDelayMS = 50;
    blueToothPI.Address = Address;
    blueToothPI.printerCmdtype = self.CurrentPrinterCmdType;
    [_CurrentPrinter setPrinterPi:blueToothPI];
    [_CurrentPrinter Open];
}

- (void)AddConnectObserver:(id)observer selector:(SEL)aSelector{
    [[NSNotificationCenter defaultCenter] addObserver:observer selector:aSelector name:(NSString *)PrinterConnectedNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:observer selector:aSelector name:(NSString *)PrinterDisconnectedNotification object:nil];
}


/*
-(void)reconnectble:(NSString *) uuid{
    BlueToothPI * _blueToothPI = (BlueToothPI *)_printerInterface;
    [_blueToothPI connectNoSCanByUUID:uuid];
    
}
-(void) AutoStartConnectBleNoScan:(NSString *) uuid{//conncet ble not no Scan
    _printerInterface  = [BlueToothFactory Create:BlueToothKind_Ble];
    [_CurrentPrinter setPrinterPi:_printerInterface];
    [self performSelector:@selector(reconnectble:) withObject:uuid afterDelay:3.0];
}
 */

-(NSData *) GetHeaderCmd:(Cmd *)cmd cmdtype:(PrinterCmdType)cmdtype{
    
    if (cmdtype==PrinterCmdTSC || cmdtype==PrinterCmdCPCL ){
        LabelCommonSetting *lblcommSetting = [LabelCommonSetting new];
        lblcommSetting.labelWidth = 70;
        lblcommSetting.labelHeight = 50;
        lblcommSetting.labelgap = 0;//3
        lblcommSetting.labelDriection = Direction_Forward;
        lblcommSetting.Density = 7;
        lblcommSetting.printCopies = 1;
        self.CurrentPrinter.CommonSetts = lblcommSetting;
        return [cmd GetHeaderCmd:lblcommSetting];
        
            } else{
        
        return [cmd GetHeaderCmd];
    }
}




@end
