//
//  MainViewController.m
//  PrinterExample
//
//  Created by King on 08/12/2017.
//  Copyright © 2017 Printer. All rights reserved.
//

#import "MainViewController.h"
#import "PrinterManager.h"
#import "ProgressHUD.h"
#import "WifiInterface.h"
#import "PinCmdFactory.h"
#import "BleDeviceListController.h"
#import "BlueToothFactory.h"
#import "ConnectedlistViewController.h"


#define UIColorFromHex(s) [UIColor colorWithRed:(((s & 0xFF0000) >> 16))/255.0 green:(((s &0xFF00) >>8))/255.0 blue:((s &0xFF))/255.0 alpha:1.0]

@interface MainViewController ()
{
   PrinterManager * _printerManager;
}
@end

@implementation MainViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    _Cboxinstruction.delegate = self;
    _CboxPortType.delegate = self;
    _Port = 9100;
    _Address = @"192.168.8.83";
    _lblChoiceAddress.text = NSLocalizedString(@"noconnectAndClick",nil);
    _printerManager = [PrinterManager sharedInstance];
    self.btnDisconnect.disabledColor = UIColorFromHex(0x6B7A8D);
    [UIColor darkGrayColor];
    self.btnConnect.disabledColor = UIColorFromHex(0x6B7A8D);
    //_MainTableView.tableFooterView = [[UIView alloc] initWithFrame:CGRectMake(0,0,self.view.frame.size.width,10)];
    
    _MainTableView.separatorStyle = UITableViewCellSeparatorStyleNone;
    [_printerManager AddConnectObserver:self selector:@selector(handleNotification:)];
    
    
    
    UITapGestureRecognizer *labelTapGestureRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(lblChoiceAddressClick)];
    [_lblChoiceAddress addGestureRecognizer:labelTapGestureRecognizer];
    _lblChoiceAddress.userInteractionEnabled = YES;
    [self setConnectStatus:false];

}

-(void)setConnectStatus:(BOOL)isConnect{
    self.btnDisconnect.enabled = isConnect;
    self.btnConnect.enabled = !isConnect;
}


-(void)UnregisterObserver{
  [[NSNotificationCenter defaultCenter] removeObserver:self];

}

-(void)dealloc {
    [self UnregisterObserver];
#if !__has_feature(objc_arc)
    [super dealloc];
#endif
}



- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}
#pragma handleNotification
- (void)handleNotification:(NSNotification *)notification{
    dispatch_async(dispatch_get_main_queue(),^{
        [ProgressHUD dismiss];
        
       // NSLog(@"notification.object=%s",object_getClassName(notification.object));
        if([notification.name isEqualToString:(NSString *)PrinterConnectedNotification])
        {

            if ([notification.object isKindOfClass:[ObserverObj class]]) {
                
                ObserverObj *obj= notification.object;
                NSLog(@"notification.object.Msgobj=%s",object_getClassName(obj.Msgobj));
                NSLog(@"notification.object.Msgvalue=%@",(NSString *)obj.Msgvalue);
                
                if ([obj.Msgobj isKindOfClass:[PrinterInterface class]]) {
                    NSInteger iindex = [_printerManager.PrinterList indexOfObject:obj.Msgobj];
                    
                  if (-1 == iindex){
                      [_printerManager.PrinterList addObject:obj.Msgobj];
                    }
                    if (_printerManager.CurrentPrinter.PrinterPi==obj.Msgobj)
                        [self setConnectStatus:YES];
                }
                obj = nil;
            }
          //  [self setConnectStatus:true];
            [ProgressHUD showSuccess:@"printer connect success" Interaction:YES];
        
        }else if([notification.name isEqualToString:(NSString *)PrinterDisconnectedNotification])
        
        {
            if ([notification.object isKindOfClass:[ObserverObj class]]) {
                ObserverObj *obj= notification.object;
                if ([obj.Msgobj isKindOfClass:[PrinterInterface class]]) {
                    NSInteger index = [_printerManager.PrinterList indexOfObject:obj.Msgobj];
                  //  SLog(@"index=%d count=%d",index,[_printerManager.PrinterList count]);
                    if (index>=0){
                        [_printerManager.PrinterList removeObject:obj.Msgobj];
                    }
                    if (_printerManager.CurrentPrinter.PrinterPi==obj.Msgobj)
                        [self setConnectStatus:NO];
                }
                obj = nil;
            }
            
            [ProgressHUD showSuccess:@"printer connect Disconnected" Interaction:YES];
         //   [self setConnectStatus:false];
        }
        
        [self refreshConnectState];
        
    });
}

-(void)refreshConnectState{
    NSString *sTitle=@"";
    if ([_printerManager.CurrentPrinter IsOpen]){
        if ([_printerManager.CurrentPrinter.PrinterPi isKindOfClass:[BlueToothPI class]])
           sTitle = [NSString stringWithFormat:@"Printer is connected(%@)",_printerManager.CurrentPrinter.PrinterPi.Name];
        else
           sTitle = [NSString stringWithFormat:@"Printer is connected(%@)",_printerManager.CurrentPrinter.PrinterPi.Address];
    } else
    {
        sTitle = @"Printer is Disconnected";
    }
    self.navigationItem.title  = sTitle;
}



#pragma mark - Table view data source



#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
     _printerManager.CurrentPrinterCmdType = (PrinterCmdType)_Cboxinstruction.currentIndex;
    if([segue.identifier isEqualToString:@"ShowConnectlist"]){
       ConnectedlistViewController *destViewController = segue.destinationViewController;
        destViewController.delegate = self;
        
        
    }
}

- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string {
    NSCharacterSet *charSet = [[NSCharacterSet characterSetWithCharactersInString:@"0123456789."] invertedSet];
    NSString *filteredStr = [[string componentsSeparatedByCharactersInSet:charSet] componentsJoinedByString:@""];
    if ([string isEqualToString:filteredStr]) {
        return YES;
    }
    return NO;
}



-(void) ConnectWifi{
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:NSLocalizedString(@"inputIpAddr",nil)  message:@"" preferredStyle:UIAlertControllerStyleAlert];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField *textField){
        textField.placeholder = @"0.0.0.0";
        textField.text = @"192.168.8.81";
        textField.keyboardType = UIKeyboardTypeDecimalPad;
        textField.delegate = self;
        
    }];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField *textField) {
        textField.placeholder = @"9100";
        textField.text =[NSString stringWithFormat:@"%ld",_Port];
        textField.secureTextEntry = false;
        textField.keyboardType = UIKeyboardTypeNumberPad;
        textField.delegate = self;

    }];
    
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:^(UIAlertAction *action){
        NSLog(@"Cancel");
        
    }];
    
    
    UIAlertAction *okAction = [UIAlertAction actionWithTitle:@"Ok" style:UIAlertActionStyleDefault handler:^(UIAlertAction * action){
        UITextField *addressField = alertController.textFields[0];
        self.Address = addressField.text;
        BOOL b=NO;
        for (PrinterInterface *pi in _printerManager.PrinterList) {
            if ([pi.Address isEqualToString:self.Address])
                b = YES;
                
        }
        
        UITextField * portField = alertController.textFields[1];
        self.Port = [portField.text integerValue];
        _lblChoiceAddress.text =  [NSString stringWithFormat:@"%@:%ld",self.Address,self.Port];
        [self setConnectStatus:b];
        

    }];
    
    
    [alertController addAction:cancelAction];
    [alertController addAction:okAction];
    
    [self presentViewController:alertController animated:YES completion:nil];
    
}

-(void) ConnectBle{
    BleDeviceListController *controller = [[BleDeviceListController alloc] init];
    controller.delegate = self;
    [[self navigationController] pushViewController:controller animated:YES];
}

-(void)selectDeviceInfo:(DeviceInfo *)device{
    if (![self.Address isEqualToString:device.UUID]){
      [self setConnectStatus:NO];
    }
    self.Address = device.UUID;
    _lblChoiceAddress.text = [NSString stringWithFormat:@"%@[%@]",device.name,device.ShortAddress];
    
    
}
-(void)selectPrinterInterface:(PrinterInterface *)printerpi{
    self.Address = printerpi.Address;
    if ([printerpi isKindOfClass:[BlueToothPI class]])
    {
        _CboxPortType.currentIndex = PrinterPortBle;
        _lblChoiceAddress.text = [NSString stringWithFormat:@"%@[%@]",printerpi.Name,printerpi.getShortAddress];
    }
    else{
      _lblChoiceAddress.text = [NSString stringWithFormat:@"%@:%d",printerpi.Address,printerpi.Port];
      _CboxPortType.currentIndex = PrinterPortWifi;
    }
    _Cboxinstruction.currentIndex = printerpi.printerCmdtype;
    [_printerManager.CurrentPrinter setPrinterPi:printerpi];
    [_printerManager setCurrentPrinterCmdType:printerpi.printerCmdtype];
    [self setConnectStatus:printerpi.IsOpen];
    [self refreshConnectState];

}


-(void) ConnectMFI{
    
}


- (void)lblChoiceAddressClick {
    switch (_CboxPortType.currentIndex) {
        case PrinterPortWifi:
            [self ConnectWifi];
            break;
        case PrinterPortBle:
            [self ConnectBle];
            break;
        case PrinterPortMFI:
           // [self ConnectBle];
            break;
        default:
            break;
    }
    
}


- (IBAction)btnConnect:(id)sender {
    if ([self.Address isEqualToString:@""] ) {
      [ProgressHUD showError:@"no set Address" Interaction:false];
        return;
    }
    
    switch (_CboxPortType.currentIndex) {
        case PrinterPortWifi:
            [_printerManager DoConnectwifi:self.Address Port:self.Port];
            break;
        case PrinterPortBle:
           // [_printerManager AutoStartConnectBleNoScan:self.Address];
            [_printerManager DoConnectBle:self.Address];
           
            break;
        case PrinterPortMFI:
           
            break;
        default:
            break;
    }

    

}
- (IBAction)btnDisConnect:(id)sender {
    [_printerManager.CurrentPrinter Close];
}

-(void)exectime{
NSLog(@"timer");
}
- (IBAction)btntest:(id)sender {

  /*
    for (PrinterInterface *pi in _printerManager.PrinterList) {//多台打印机同时打印(Multiple printers print at the same time)
        Cmd *cmd =  [_printerManager CreateCmdClass:pi.printerCmdtype];
        if (cmd == NULL)
            return;
        [cmd Clear];
        [cmd Append:cmd.GetSelftestCmd];
        NSLog(@"data=%@",[cmd GetCmd]);
        if (pi.IsOpen){
            [pi Write:[cmd GetCmd]];
            
        }
    }
  */
    
    if (_printerManager.CurrentPrinter.IsOpen){
        Cmd *cmd =  [_printerManager CreateCmdClass:(PrinterCmdType)_Cboxinstruction.currentIndex];
        if (cmd == NULL)
            return;
        [cmd Clear];
        [cmd Append:cmd.GetSelftestCmd];
        NSLog(@"data=%@",[cmd GetCmd]);
        if ([_printerManager.CurrentPrinter IsOpen]){
            [_printerManager.CurrentPrinter Write:[cmd GetCmd]];
        }
    
    }
}

 


-(void)WBComboValueChanged:(id)sender{
    _printerManager.CurrentPrinterCmdType = (PrinterCmdType)_Cboxinstruction.currentIndex;
    _printerManager.CurrentPrinterPortType = (PrinterPortType)_CboxPortType.currentIndex;
}




@end
