//
//  PrinterManager.m
//  PrinterExample
//
//  Created by 朱正锋 on 12/12/2017.
//  Copyright © 2017 Printer. All rights reserved.
//

#import "PrinterManager.h"

#import "WIFIFactory.h"
#import "PinPrinterFactory.h"
#import "ESCFactory.h"
#import "ThermalPrinter.h"
#import "ThermalPrinterFactory.h"
#import "LabelPrinterFactory.h"
#import "PinCmdFactory.h"
#import "BleDevice.h"
#import "BleService.h"


@implementation PrinterManager

+ (PrinterManager *)sharedInstance{
    static PrinterManager *_sharedInstance = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        _sharedInstance = [[PrinterManager alloc] init];
        _sharedInstance.CurrentPrinterPortType = PrinterPortWifi;
        _sharedInstance.CurrentPrinterCmdType = PrinterCmdESC;
        _sharedInstance.CurrentPrinter = [_sharedInstance CreatePrinterClass:PrinterCmdESC];
        
    });
    
    return _sharedInstance;
}

-(Cmd *)CreateCmdClass:(PrinterCmdType)PrinterCmdType{
    Cmd *cmd;
    switch (PrinterCmdType) {
        case PrinterCmdESC:
            cmd =  [ESCFactory Create];
            break;
        case PrinterCmdTSC:
            //cmd=  [TSCFactory Create];
            break;
        case PrinterCmdCPCL:
            // cmd  [CPCLFactory Create];
            break;
        case PrinterCmdPIN:
            cmd = [PinCmdFactory Create];
            break;
            
        default:
            cmd=nil;
    }
    return cmd;
}


-(Printer *)CreatePrinterClass:(PrinterCmdType)PrinterCmdType{
    Printer * _printer;
    switch (PrinterCmdType) {
        case PrinterCmdESC:
            _printer =  [ThermalPrinterFactory Create];
            break;
        case PrinterCmdTSC:
        case PrinterCmdCPCL:
            _printer =  [LabelPrinterFactory Create];
            break;
        case PrinterCmdPIN:
            _printer = [PinPrinterFactory Create];
            break;
        default:
            _printer=nil;
    }
    return _printer;
}


-(void)DoConnectwifi:(PrinterCmdType)PrinterCmdType Address:(NSString *)Address Port:(NSInteger)Port {
    _printerInterface = [WIFIFactory Create];
    _printerInterface.Address = Address;
    _printerInterface.Port = Port;
    [_CurrentPrinter setPrinterPi:_printerInterface];
    [_CurrentPrinter Open];
}

- (void)AddConnectObserver:(id)observer selector:(SEL)aSelector{
    [[NSNotificationCenter defaultCenter] addObserver:observer selector:aSelector name:(NSString *)PrinterConnectedNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:observer selector:aSelector name:(NSString *)PrinterDisconnectedNotification object:nil];
  //  PrinterConfigureChangeNotification
    
}


-(NSMutableArray *)getBleNameList{
    NSMutableArray * list =  [[NSMutableArray alloc] init];
    
    for( BleDevice * device in [BleService sharedInstance].bleDevices){
        [list addObject:device.name];
    }
    
    return list;
}

-(NSMutableArray *)getBleUuidList{
    NSMutableArray * list =  [[NSMutableArray alloc] init];
    
    for( BleDevice * device in [BleService sharedInstance].bleDevices){
        [list addObject:device.UUID];
    }
    
    return list;
}


@end
