//
//  BleService.h
//  SportStone
//
//  Created by  Andrew Huang on 15/1/5.
//  Copyright (c) 2015年  Andrew Huang. All rights reserved.
//
#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
#if TARGET_OS_IPHONE
    #import <CoreBluetooth/CoreBluetooth.h>
#else
    #import <IOBluetooth/IOBluetooth.h>
#endif

#define BLE_CURRENT_UUID @"ble_current_uuid"
#define BLE_CURRENT_NAME @"ble_current_name"
#define BLE_CURRENT_ADDRESS @"ble_current_address"

typedef enum {
    BleScanComplete,
    BleScanDevice,
}BleServiceStatus;

//typedef enum {
//    
//    BleWorkState , //工作状态，即联接上工作
//    BleSelectState, //选择状态，即在选择设备列表，分两个状态主要在工作状态要主动去联接已经配置设备
//}BleScanState;

#import "BleDevice.h"
#import "BleProtocol.h"





@interface BleService : NSObject<CBCentralManagerDelegate>


@property (nonatomic, strong) CBCentralManager * bleManager;
@property (nonatomic, strong) NSMutableArray *   bleDevices;
@property (nonatomic, strong) NSMutableArray *  connSuccDevices;//己经连接成功的蓝牙设备列表

@property (nonatomic, strong) NSString *   currentUUID;

@property (nonatomic, strong) NSString *   bindUUID;
@property (nonatomic, strong) NSString *   bindName;
@property (nonatomic) BOOL singleDeviceMode; //一次只联接一个设备

@property (nonatomic, strong) dispatch_queue_t bleQueue;



@property (strong ,nonatomic) NSTimer *rssiTimer;  //统一rssiy读取

//@property (nonatomic) BleScanState  scanState;

//@property (nonatomic, strong) CBPeripheral * currentPeripheral;

@property (nonatomic, strong) BleDevice * currentBleDevice;

@property (nonatomic, weak) BleProtocol * protocol;


@property (strong,nonatomic) NSMutableArray *nRingtones;

@property (nonatomic, strong) NSString  * prefixName; //设备前缀名称，用于判断是否是本公司产品
//串口类设备列表,包含串口服务名,接收字符名,发送字符名,它来源于plist文件
//一些运算后的值,比如对应CBUUID值也保存在此
@property (nonatomic, strong) NSMutableArray * serialServices;

//@property (nonatomic, strong) NSString * serialServiceUUID; //串口服务uuid
//@property (nonatomic, strong) NSString * serialWriteUUID; //串口服务对应写通道
//@property (nonatomic, strong) NSString * serialReadUUID; //串口服务对应读通道
//
//
//@property (nonatomic, strong) CBUUID * seriaiServiceCBUUID ;
//@property (nonatomic, strong) CBUUID * seriaiReadCBUUID;
//@property (nonatomic, strong) CBUUID * seriaiWriteCBUUID;

@property (nonatomic,strong) Class deviceClass;



- (id)initWithIdentifier:(NSString *)theIdentifier;

-(void)startBle;
-(void)stop;

-(void)pauseRssiTimer;
-(void)resumeRssiTimer;

//扫描所有ble 设备
-(BOOL)startScanAll:(NSInteger)scanSeconds clear:(BOOL)clear;

//扫描指定服务的ble 设备
-(BOOL)startScan:(NSInteger)scanSeconds serviceArray:(NSArray *)uuidArray clear:(BOOL)clear;

//扫描指定服务的ble 设备
-(BOOL)startScanByUuid:(int)scanSeconds ;



//联接指定设备
-(BOOL)connect:(CBPeripheral *) peripheral;

-(BOOL)connectByUUID:(NSString  *) uuid;

//断开指定设备
-(BOOL)disconnect:(CBPeripheral *)peripheral;





-(BleDevice *)bindByUUID:(NSString *)uuid;
-(BOOL)unbind:(BleDevice *)device;
-(BOOL)bind:(BleDevice *)device;
-(BOOL)isBind:(BleDevice *)device;
-(BOOL)isBind;
-(BleDevice *)getBindDevice;




-(BleDevice *)createDevice;

-(void)saveCurrentDevice:(NSString *)uuid;
-(void)selectDevice:(BleDevice *)device;

-(NSString *)getBluetoothRssi;
-(BleDevice *)findDeviceByUUID:(NSString *) uuid;

-(void)stopScan;

+ (BleService *)sharedInstance;
-(int)findconnSuccDevicesDevice:(BleDevice *) device; //查找已连接的设备
-(void)RemoveSuccDevicesDevice:(BleDevice *) device;

@end
