//
//  UIImage+Ext.h
//  RTPrinter
//
//  Created by PRO on 16/1/6.
//  Copyright © 2016年 bluedrum. All rights reserved.
// http://blog.csdn.net/xuhuan_wh/article/details/6434055
//

#import <UIKit/UIKit.h>

//00 01 - 49 00 - 12 00 - 78 00
//00 03 - 20 00 - 01 00 - 00 00

typedef struct PcxHeader
{
    Byte Identifier;        // PCX Id Number (Always 0x0A) // 10
    Byte Version;           // Version Number    // 5
    Byte Encoding;          // Encoding Format    // 1
    Byte BitsPerPixel;      // Bits per Pixel    // 8
    unsigned short XStart;            // Left of image     // 0
    unsigned short YStart;            // Top of Image     // 0
    unsigned short XEnd;              // Right of Image    // 319
    unsigned short YEnd;              // Bottom of image    // 199
    unsigned short HorzRes;           // Horizontal Resolution   // 320
    unsigned short VertRes;           // Vertical Resolution   // 200
    Byte Palette[48];       // 16-Color EGA Palette
    Byte Reserved1;         // Reserved (Always 0)
    Byte NumBitPlanes;      // Number of Bit Planes   // 1
    unsigned short BytesPerLine;      // Bytes per Scan-line   // 320
    unsigned short PaletteType;       // Palette Type     // 0
    unsigned short HorzScreenSize;    // Horizontal Screen Size   // 0
    unsigned short VertScreenSize;    // Vertical Screen Size   // 0
    Byte Reserved2[54];     // Reserved (Always 0)
}PCX_HEADER;

#define COLOR_RGB_TO_INT(r,g,b) ( 0xFF000000 |  (((r) & 0xFF)<<16) | (((g) & 0xFF)<<8) | ( (b) & 0xFF)  )
#define ARGB_TO_COLOR(a,r,g,b) ( (((a) & 0xFF)<<24) | (((r) & 0xFF)<<16) | (((g) & 0xFF)<<8) | (( (b) & 0xFF)) )

#define COLOR_TO_RED(rgb) ((rgb & 0x00FF0000) >> 16)
#define COLOR_TO_GREEN(rgb) ((rgb & 0x0000FF00) >> 8)
#define COLOR_TO_BLUE(rgb) ((rgb & 0x000000FF) >> 0)
#define COLOR_TO_ALPHA(rgb) ((rgb & 0xFF000000) >> 24)


@interface UIImageExt : NSObject 

//1.等比率缩放
+ (UIImage *)scaleImage:(UIImage *)image toScale:(float)scaleSize;

//2.自定长宽
+ (UIImage *)scaleImage:(UIImage *)image toSize:(CGSize)reSize;
//3.等比缩放到指定宽度
+ (UIImage *)scaleImage:(UIImage *)image toWidth:(float)width;

//4.等比缩放到指定高度
+ (UIImage *)scaleImage:(UIImage *)image toHeight:(float)height;

//3.截图特定View
//只要是继承UIView的object 都可以处理
//必须先import QuzrtzCore.framework
//1. 从UIView中获取图像相当于窗口截屏。

//(ios提供全局的全屏截屏函数UIGetScreenView(). 如果需要特定区域的图像，可以crop一下)
/*
CGImageRef screen = UIGetScreenImage();
UIImage* image = [UIImage imageWithCGImage:screen];
*/
+(UIImage*)captureView:(UIView *)theView;

//截完整的Web图像,captureView只是截可视部分
+(UIImage *)captureWeb:(UIWebView *)webView;


+(UIImage *)grayImage1bpp:(UIImage *)image ;
//生成灰色图像
+(UIImage *)grayImage:(UIImage *)image ;

//生成RGB点阵,freeWhenDone=YES表示随返回NSData自动释放,否则只能调用free()手动释放
+(NSData *)rgbBuffer:(UIImage *)image ;
//预留包头长度 headerLen
+(NSData *)rgbBuffer:(UIImage *)image headerLen:(NSInteger)headerLen ;

//用通用算法求得灰化数组
+(NSData *)grayBuffer:(UIImage *)image zeroCopy:(BOOL)zeroCopy;

+(NSData *)grayBuffer:(UIImage *)image headerLen:(NSInteger)headerLen zeroCopy:(BOOL)zeroCopy;

+(NSString *)typeForImageData:(NSData *)data;


+ (BOOL)writeImage:(UIImage*)image toFileAtPath:(NSString*)aPath;

//根据灰度数据生成相应 PCX 格式数据
+(NSData *)pcxGrayImage:(UIImage *)grayImage;


+(NSString *)writePngImage:(UIImage *)image withName:(NSString *)name;



@end
